import threading
from collections.abc import Callable, Generator, Iterator
from threading import Thread
from typing import Any, Generic, TypeVar

from typing_extensions import override

__all__ = (
    "Local",
    "LocalManager",
    "LocalStack",
    "bgThread",
    "default_socket_timeout",
    "get_ident",
)

_T = TypeVar("_T")

def get_ident() -> int: ...
def default_socket_timeout(timeout: float | None) -> Generator[None, None, None]: ...

class bgThread(Thread):
    daemon: bool
    name: str
    def __init__(self, name: str | None = ..., **kwargs: Any) -> None: ...
    def body(self) -> None: ...
    def on_crash(self, msg: Any, *fmt: Any, **kwargs: Any) -> None: ...
    @override
    def run(self) -> None: ...
    def stop(self) -> None: ...

class Local:
    __storage__: dict[int, dict[str, Any]]
    __ident_func__: Callable[[], int]

    def __init__(self) -> None: ...
    def __iter__(self) -> Iterator[tuple[str, Any]]: ...
    def __call__(self, proxy: Any) -> Any: ...
    def __release_local__(self) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    @override
    def __setattr__(self, name: str, value: Any) -> None: ...
    @override
    def __delattr__(self, name: str) -> None: ...

class _LocalStack(Generic[_T]):
    @property
    def __ident_func__(self) -> Callable[[], int]: ...
    def __init__(self) -> None: ...
    def __release_local__(self) -> None: ...
    def __call__(self) -> Any: ...
    def push(self, obj: _T) -> list[_T]: ...
    def pop(self) -> _T: ...
    def __len__(self) -> int: ...
    @property
    def stack(self) -> list[_T]: ...
    @property
    def top(self) -> _T: ...

class LocalManager:
    locals: Any
    ident_func: Any
    def __init__(
        self, locals: Any | None = ..., ident_func: Any | None = ...
    ) -> None: ...
    def get_ident(self) -> int: ...
    def cleanup(self) -> None: ...

class _FastLocalStack(threading.local, Generic[_T]):
    stack: list[_T]
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    @property
    def top(self) -> _T: ...

LocalStack: type[_FastLocalStack[Any] | _LocalStack[Any]]



from collections.abc import Callable, Generator
from logging import Handler, Logger
from typing import Any, TypeVar

from amqp import promise as promise
from typing_extensions import override

_T = TypeVar("_T")

class NullHandler(Handler):
    @override
    def emit(self, record: Any) -> None: ...

def bytes_to_str(s: bytes | str) -> str: ...
def str_to_bytes(s: str | bytes) -> bytes: ...
def get_logger(logger: Logger | str) -> Logger: ...
def set_cloexec(fd: int, cloexec: bool) -> None: ...
def coro(gen: Callable[..., Generator[_T, Any, Any]]) -> Callable[..., _T]: ...



import pickle
from signal import _HANDLER, _SIGNUM, Signals
from typing import Any

from billiard.exceptions import RestartFreqExceeded as _RestartFreqExceeded

pickle_load = pickle.load

def pickle_loads(s: Any, load: Any = ...) -> Any: ...

SIGMAP: dict[Signals, str]
TERM_SIGNAL: Signals
TERM_SIGNAME: str
REMAP_SIGTERM: str | None
TERMSIGS_IGNORE: set[str]
TERMSIGS_FORCE: set[str]
EX_SOFTWARE: int
TERMSIGS_DEFAULT: set[str]
TERMSIGS_FULL: set[str]

def human_status(status: int) -> str: ...
def maybe_setsignal(signum: _SIGNUM, handler: _HANDLER) -> None: ...
def signum(sig: int) -> _SIGNUM: ...
def reset_signals(handler: _HANDLER = ..., full: bool = ...) -> None: ...

class restart_state:
    RestartFreqExceeded = _RestartFreqExceeded
    def __init__(self, maxR: int, maxT: int) -> None: ...
    R: int
    T: float | None
    def step(self, now: float | None = ...) -> None: ...



from collections.abc import Callable as _Callable
from typing import Any, TypeVar

__all__ = ("Callable", "Property", "warn")

_F = TypeVar("_F", bound=_Callable[..., Any])

DEPRECATION_FMT: str
PENDING_DEPRECATION_FMT: str

class CDeprecationWarning(UserWarning): ...
class CPendingDeprecationWarning(PendingDeprecationWarning): ...

def warn(
    description: str | None = None,
    deprecation: str | None = None,
    removal: str | None = None,
    alternative: str | None = None,
    stacklevel: int = 2,
) -> None: ...
def Callable(
    deprecation: str | None = None,
    removal: str | None = None,
    alternative: str | None = None,
    description: str | None = None,
) -> _Callable[[_F], _F]: ...
def Property(
    deprecation: str | None = None,
    removal: str | None = None,
    alternative: str | None = None,
    description: str | None = None,
) -> _Callable[[_F], _F]: ...



from typing import NamedTuple

class method_t(NamedTuple):
    method_sig: tuple[int, int]
    args: str | None
    content: bool

def method(
    method_sig: tuple[int, int], args: str | None = ..., content: bool = ...
) -> method_t: ...

class Connection:
    CLASS_ID: int
    Start: tuple[int, int]
    StartOk: tuple[int, int]
    Secure: tuple[int, int]
    SecureOk: tuple[int, int]
    Tune: tuple[int, int]
    TuneOk: tuple[int, int]
    Open: tuple[int, int]
    OpenOk: tuple[int, int]
    Close: tuple[int, int]
    CloseOk: tuple[int, int]
    Blocked: tuple[int, int]
    Unblocked: tuple[int, int]

class Channel:
    CLASS_ID: int
    Open: tuple[int, int]
    OpenOk: tuple[int, int]
    Flow: tuple[int, int]
    FlowOk: tuple[int, int]
    Close: tuple[int, int]
    CloseOk: tuple[int, int]

class Exchange:
    CLASS_ID: int
    Declare: tuple[int, int]
    DeclareOk: tuple[int, int]
    Delete: tuple[int, int]
    DeleteOk: tuple[int, int]
    Bind: tuple[int, int]
    BindOk: tuple[int, int]
    Unbind: tuple[int, int]
    UnbindOk: tuple[int, int]

class Queue:
    CLASS_ID: int
    Declare: tuple[int, int]
    DeclareOk: tuple[int, int]
    Bind: tuple[int, int]
    BindOk: tuple[int, int]
    Unbind: tuple[int, int]
    UnbindOk: tuple[int, int]
    Purge: tuple[int, int]
    PurgeOk: tuple[int, int]
    Delete: tuple[int, int]
    DeleteOk: tuple[int, int]

class Basic:
    CLASS_ID: int
    Qos: tuple[int, int]
    QosOk: tuple[int, int]
    Consume: tuple[int, int]
    ConsumeOk: tuple[int, int]
    Cancel: tuple[int, int]
    CancelOk: tuple[int, int]
    Publish: tuple[int, int]
    Return: tuple[int, int]
    Deliver: tuple[int, int]
    Get: tuple[int, int]
    GetOk: tuple[int, int]
    GetEmpty: tuple[int, int]
    Ack: tuple[int, int]
    Reject: tuple[int, int]
    RecoverAsync: tuple[int, int]
    Recover: tuple[int, int]
    RecoverOk: tuple[int, int]
    Nack: tuple[int, int]

class Tx:
    CLASS_ID: int
    Select: tuple[int, int]
    SelectOk: tuple[int, int]
    Commit: tuple[int, int]
    CommitOk: tuple[int, int]
    Rollback: tuple[int, int]
    RollbackOk: tuple[int, int]

class Confirm:
    CLASS_ID: int
    Select: tuple[int, int]
    SelectOk: tuple[int, int]



from kombu.asynchronous.hub import Hub

def callback_for(h: Hub, fd: int, flag: int, *default: str) -> str: ...
def repr_flag(flag: int) -> str: ...
def repr_active(h: Hub) -> str: ...
def repr_events(h: Hub, events: list[tuple[int, int]]) -> str: ...
def repr_readers(h: Hub) -> list[str]: ...
def repr_writers(h: Hub) -> list[str]: ...



from typing import Any

from celery.security.certificate import Certificate, CertStore
from celery.security.key import PrivateKey

__all__ = ("SecureSerializer", "register_auth")

class SecureSerializer:
    def __init__(
        self,
        key: PrivateKey | None = None,
        cert: Certificate | None = None,
        cert_store: CertStore | None = None,
        digest: str = "sha256",
        serializer: str = "json",
    ) -> None: ...
    def serialize(self, data: Any) -> bytes: ...
    def deserialize(self, data: bytes) -> Any: ...

def register_auth(
    key: str | None = None,
    key_password: str | None = None,
    cert: str | None = None,
    store: str | None = None,
    digest: str = "sha256",
    serializer: str = "json",
) -> SecureSerializer: ...



from collections.abc import Callable
from typing import Any

from amqp.basic_message import Message as Message
from amqp.exceptions import UnexpectedFrame as UnexpectedFrame

__all__ = ("frame_handler", "frame_writer")

FRAME_OVERHEAD: int

def frame_handler(
    connection: Any,
    callback: Callable[..., Any],
    unpack_from: Callable[..., tuple[Any, ...]] = ...,
    content_methods: frozenset[tuple[int, int]] = ...,
) -> Callable[..., Any]: ...
def frame_writer(
    connection: Any,
    transport: Any,
    pack: Callable[..., bytes] = ...,
    pack_into: Callable[..., None] = ...,
    range: type[range] = ...,
    len: Callable[[Any], int] = ...,
    bytes: type[bytes] = ...,
    str_to_bytes: Callable[[str], bytes] = ...,
    text_t: type[str] = ...,
) -> Callable[..., Any]: ...
def str_to_bytes(s: str) -> bytes: ...



from typing import IO, Any

import click
from celery.bin.base import CeleryCommand

UNABLE_TO_LOAD_APP_APP_MISSING: str
UNABLE_TO_LOAD_APP_ERROR_OCCURRED: str
UNABLE_TO_LOAD_APP_MODULE_NOT_FOUND: str
WRONG_APP_OPTION_USAGE_MESSAGE: str

def main() -> int: ...
def previous_show_implementation(self: Any, file: IO[str] | None = None) -> None: ...

celery: click.Group
report: CeleryCommand



from typing import Any, ClassVar

from celery.bootsteps import Step
from typing_extensions import override

__all__ = ("Agent",)

class Agent(Step):
    name: ClassVar[str]  # pyright: ignore[reportIncompatibleVariableOverride]

    def __init__(self, c: Any, **kwargs: Any) -> None: ...
    @override
    def create(self, c: Any) -> Any: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]



from collections.abc import Callable
from typing import Any

__all__ = ("Event", "event_exchange", "get_exchange", "group_from")

event_exchange: Any

def Event(
    type: str,
    _fields: dict[str, Any] | None = None,
    __dict__: type[dict[str, Any]] = ...,
    __now__: Callable[[], float] = ...,
    **fields: Any,
) -> dict[str, Any]: ...
def group_from(type: str) -> str: ...
def get_exchange(conn: Any, name: str = ...) -> Any: ...



import re
from logging import Logger
from typing import Any

from kombu.transport.virtual import Channel as VirtualChannel
from kombu.transport.virtual import QoS as VirtualQoS
from kombu.transport.virtual import Transport as VirtualTransport
from kombu.utils.objects import cached_property
from typing_extensions import override

logger: Logger
CHARS_REPLACE_TABLE: dict[int, int]
SQS_MAX_MESSAGES: int

def maybe_int(x: Any) -> int | Any: ...

class UndefinedQueueException(Exception): ...
class InvalidQueueException(Exception): ...
class AccessDeniedQueueException(Exception): ...
class DoesNotExistQueueException(Exception): ...

class QoS(VirtualQoS):
    @override
    def reject(self, delivery_tag: str, requeue: bool = ...) -> None: ...
    def apply_backoff_policy(
        self,
        routing_key: str,
        delivery_tag: str,
        backoff_policy: dict[int, int],
        backoff_tasks: list[str],
    ) -> None: ...
    def extract_task_name_and_number_of_retries(
        self, delivery_tag: str
    ) -> tuple[str, int]: ...

_QoS = QoS

class Channel(VirtualChannel):
    default_region: str
    default_visibility_timeout: int
    default_wait_time_seconds: int
    domain_format: str
    _asynsqs: Any | None
    _predefined_queue_async_clients: dict[str, Any]
    _sqs: Any | None
    _predefined_queue_clients: dict[str, Any]
    _queue_cache: dict[str, str]
    _noack_queues: set[str]
    QoS: type[_QoS]  # pyright: ignore[reportIncompatibleVariableOverride]
    B64_REGEX: re.Pattern[bytes]

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @override
    def basic_consume(
        self, queue: str, no_ack: bool, *args: Any, **kwargs: Any
    ) -> str: ...
    @override
    def basic_cancel(self, consumer_tag: str) -> Any: ...
    @override
    def drain_events(
        self, timeout: float | None = ..., callback: Any | None = ..., **kwargs: Any
    ) -> None: ...
    def entity_name(self, name: str, table: dict[int, int] = ...) -> str: ...
    def canonical_queue_name(self, queue_name: str) -> str: ...
    @override
    def basic_ack(self, delivery_tag: str, multiple: bool = ...) -> None: ...
    @override
    def close(self) -> None: ...
    def new_sqs_client(
        self,
        region: str,
        access_key_id: str,
        secret_access_key: str,
        session_token: str | None = ...,
    ) -> Any: ...
    def sqs(self, queue: str | None = ...) -> Any: ...
    def asynsqs(self, queue: str | None = ...) -> Any: ...
    def generate_sts_session_token(
        self, role_arn: str, token_expiry_seconds: int
    ) -> dict[str, Any]: ...
    def generate_sts_session_token_with_buffer(
        self, role_arn: str, token_expiry_seconds: int, token_buffer_seconds: int = ...
    ) -> dict[str, Any]: ...
    @property
    def conninfo(self) -> Any: ...
    @property
    def transport_options(self) -> dict[str, Any]: ...
    @cached_property
    def visibility_timeout(self) -> int: ...
    @cached_property
    def predefined_queues(self) -> dict[str, Any]: ...
    @cached_property
    def queue_name_prefix(self) -> str: ...
    @cached_property
    def supports_fanout(self) -> bool: ...  # pyright: ignore[reportIncompatibleVariableOverride]
    @cached_property
    def region(self) -> str: ...
    @cached_property
    def regioninfo(self) -> Any: ...
    @cached_property
    def is_secure(self) -> bool | None: ...
    @cached_property
    def port(self) -> int | None: ...
    @cached_property
    def endpoint_url(self) -> str | None: ...
    @cached_property
    def wait_time_seconds(self) -> int: ...
    @cached_property
    def sqs_base64_encoding(self) -> bool: ...
    @cached_property
    def fetch_message_attributes(self) -> Any: ...
    @property
    def get_message_attributes(self) -> dict[str, Any]: ...

_Channel = Channel

class Transport(VirtualTransport):
    Channel: type[_Channel]  # pyright: ignore[reportIncompatibleVariableOverride]

    polling_interval: int  # pyright: ignore[reportIncompatibleVariableOverride]
    wait_time_seconds: int
    default_port: None  # pyright: ignore[reportIncompatibleVariableOverride]
    connection_errors: tuple[type[Exception], ...]  # pyright: ignore[reportIncompatibleVariableOverride]
    channel_errors: tuple[type[Exception], ...]  # pyright: ignore[reportIncompatibleVariableOverride]
    driver_type: str
    driver_name: str
    implements: Any

    @property
    @override
    def default_connection_params(self) -> dict[str, Any]: ...



__all__ = (
    "AMQPDeprecationWarning",
    "AMQPError",
    "AMQPNotImplementedError",
    "AccessRefused",
    "ChannelError",
    "ChannelNotOpen",
    "ConnectionError",
    "ConnectionForced",
    "ConsumerCancelled",
    "ContentTooLarge",
    "FrameError",
    "FrameSyntaxError",
    "InternalError",
    "InvalidCommand",
    "InvalidPath",
    "IrrecoverableChannelError",
    "IrrecoverableConnectionError",
    "MessageNacked",
    "NoConsumers",
    "NotAllowed",
    "NotFound",
    "PreconditionFailed",
    "RecoverableChannelError",
    "RecoverableConnectionError",
    "ResourceError",
    "ResourceLocked",
    "UnexpectedFrame",
)

class AMQPError(Exception):
    code: int
    recoverable: bool
    reply_text: str | None
    method_sig: tuple[int, int] | None
    method_name: str | None
    reply_code: int | None

    def __init__(
        self,
        reply_text: str | None = ...,
        method_sig: tuple[int, int] | None = ...,
        method_name: str | None = ...,
        reply_code: int | None = ...,
    ) -> None: ...
    @property
    def method(self) -> tuple[int, int] | None: ...

class ConnectionError(AMQPError): ...
class ChannelError(AMQPError): ...
class RecoverableConnectionError(ConnectionError): ...
class IrrecoverableConnectionError(ConnectionError): ...
class RecoverableChannelError(ChannelError): ...
class IrrecoverableChannelError(ChannelError): ...

class ConsumerCancelled(RecoverableConnectionError):
    code: int

class ContentTooLarge(RecoverableChannelError):
    code: int

class NoConsumers(RecoverableChannelError):
    code: int

class ConnectionForced(RecoverableConnectionError):
    code: int

class InvalidPath(IrrecoverableConnectionError):
    code: int

class AccessRefused(IrrecoverableChannelError):
    code: int

class NotFound(IrrecoverableChannelError):
    code: int

class ResourceLocked(RecoverableChannelError):
    code: int

class PreconditionFailed(IrrecoverableChannelError):
    code: int

class FrameError(IrrecoverableConnectionError):
    code: int

class FrameSyntaxError(IrrecoverableConnectionError):
    code: int

class InvalidCommand(IrrecoverableConnectionError):
    code: int

class ChannelNotOpen(IrrecoverableConnectionError):
    code: int

class UnexpectedFrame(IrrecoverableConnectionError):
    code: int

class ResourceError(RecoverableConnectionError):
    code: int

class NotAllowed(IrrecoverableConnectionError):
    code: int

class AMQPNotImplementedError(IrrecoverableConnectionError):
    code: int

class InternalError(IrrecoverableConnectionError):
    code: int

class MessageNacked(Exception): ...
class AMQPDeprecationWarning(UserWarning): ...

def error_for_code(
    code: int,
    text: str,
    method: tuple[int, int],
    default: type[AMQPError],
) -> AMQPError: ...



from collections.abc import Callable, MutableSet
from types import ModuleType
from typing import Any

from celery.utils.collections import LimitedSet

__all__ = (
    "SOFTWARE_INFO",
    "Persistent",
    "active_requests",
    "maybe_shutdown",
    "reserved_requests",
    "revoked",
    "task_accepted",
    "task_ready",
    "task_reserved",
    "total_count",
)

class Persistent:
    db: Any
    merge: Callable[..., Any]
    open: Callable[..., Any]
    protocol: int
    storage: ModuleType
    # These are zlib.compress and zlib.decompress functions assigned as class attributes
    compress: Callable[..., bytes]
    decompress: Callable[..., bytes]

    def __init__(self, state: Any, filename: str, clock: Any | None = None) -> None: ...
    def save(self) -> None: ...
    def close(self) -> None: ...
    def sync(self) -> None: ...

C_BENCH: None
C_BENCH_EVERY: int
REVOKES_MAX: int
REVOKE_EXPIRES: float
SUCCESSFUL_EXPIRES: float
SUCCESSFUL_MAX: int
SOFTWARE_INFO: dict[str, str]

revoked: LimitedSet
revoked_stamps: dict[str, set[str]]
active_requests: MutableSet[Any]
reserved_requests: MutableSet[Any]
successful_requests: LimitedSet
requests: dict[str, Any]
total_count: dict[str, int]
all_total_count: list[int]

should_stop: None
should_terminate: None

def maybe_shutdown() -> None: ...
def reset_state() -> None: ...
def task_accepted(
    request: Any,
    _all_total_count: list[int] | None = None,
    add_request: Callable[..., Any] = ...,
    add_active_request: Callable[..., Any] = ...,
    add_to_total_count: Callable[..., Any] = ...,
) -> None: ...
def task_reserved(
    request: Any,
    add_request: Callable[..., Any] = ...,
    add_reserved_request: Callable[..., Any] = ...,
) -> None: ...
def task_ready(
    request: Any,
    successful: bool = False,
    remove_request: Callable[..., Any] = ...,
    discard_active_request: Callable[..., Any] = ...,
    discard_reserved_request: Callable[..., Any] = ...,
) -> None: ...



from collections.abc import Callable
from typing import Any, ClassVar

import kombu
from celery.app.base import Celery
from celery.bootsteps import Blueprint, StartStopStep
from typing_extensions import override
from vine.promises import promise

__all__ = ("Consumer", "Evloop", "dump_body")

class Evloop(StartStopStep):
    name: ClassVar[str]  # pyright: ignore[reportIncompatibleVariableOverride]
    label: ClassVar[str]  # pyright: ignore[reportIncompatibleVariableOverride]

    def patch_all(self, c: Any) -> None: ...
    @override
    def start(self, c: Any) -> None: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]

def dump_body(m: Any, body: Any) -> str: ...

class Consumer:
    app: Celery
    hostname: str
    pool: Any
    connection: kombu.Connection | None
    blueprint: Blueprint
    Blueprint: type[Blueprint]
    Strategies: type[dict[Any, Any]]
    restart_count: int
    broker_connection_retry_attempt: int
    first_connection_attempt: bool
    init_callback: Callable[..., Any] | None

    def __init__(
        self,
        on_task_request: Callable[..., Any],
        init_callback: Callable[..., Any] = ...,
        hostname: str | None = None,
        pool: Any = None,
        app: Celery | None = None,
        timer: Any = None,
        controller: Any = None,
        hub: Any = None,
        amqheartbeat: float | None = None,
        worker_options: dict[str, Any] | None = None,
        disable_rate_limits: bool = False,
        initial_prefetch_count: int = 2,
        prefetch_multiplier: int = 1,
        **kwargs: Any,
    ) -> None: ...
    def bucket_for_task(self, type: Any) -> Any: ...
    def reset_rate_limits(self) -> None: ...
    def add_task_queue(
        self,
        queue: str | kombu.Queue,
        exchange: str | kombu.Exchange | None = None,
        exchange_type: str | None = None,
        routing_key: str | None = None,
        **options: Any,
    ) -> kombu.Queue: ...
    def cancel_task_queue(self, queue: str | kombu.Queue) -> None: ...
    def apply_eta_task(self, task: Any) -> None: ...
    def call_soon(
        self,
        p: Callable[..., Any],
        *args: Any,
        **kwargs: Any,
    ) -> None: ...
    def cancel_active_requests(self) -> None: ...
    def connect(self) -> kombu.Connection: ...
    def connection_for_read(
        self, heartbeat: float | None = None
    ) -> kombu.Connection: ...
    def connection_for_write(
        self, url: str | None = None, heartbeat: float | None = None
    ) -> kombu.Connection: ...
    def create_task_handler(
        self, promise: type[promise] = ...
    ) -> Callable[..., Any]: ...
    def ensure_connected(self, conn: kombu.Connection) -> kombu.Connection: ...
    def loop_args(self) -> tuple[Any, ...]: ...
    @property
    def max_prefetch_count(self) -> int: ...
    def on_close(self) -> None: ...
    def on_connection_error_after_connected(self, exc: Exception) -> None: ...
    def on_connection_error_before_connected(self, exc: Exception) -> None: ...
    def on_decode_error(self, message: Any, exc: Exception) -> None: ...
    def on_invalid_task(self, body: Any, message: Any, exc: Exception) -> None: ...
    def on_ready(self) -> None: ...
    def on_send_event_buffered(self) -> Callable[..., Any]: ...
    def on_unknown_message(self, body: Any, message: Any) -> None: ...
    def on_unknown_task(self, body: Any, message: Any, exc: Exception) -> None: ...
    def perform_pending_operations(self) -> None: ...
    def register_with_event_loop(self, hub: Any) -> None: ...
    def stop(self) -> None: ...
    def start(self) -> None: ...
    def shutdown(self) -> None: ...
    @property
    def timer(self) -> Any: ...
    def update_strategies(self) -> None: ...



from collections.abc import Callable, Iterable
from types import ModuleType
from typing import Any

from celery.backends.asynchronous import BaseResultConsumer
from celery.backends.base import KeyValueStoreBackend
from kombu.utils.objects import cached_property
from typing_extensions import override

__all__ = ("RedisBackend", "SentinelBackend")

class RedisBackend(KeyValueStoreBackend):
    ResultConsumer: type[BaseResultConsumer]
    connection_class_ssl: type[Any]
    max_connections: int | None
    redis: Any  # redis module

    retry_policy: cached_property[dict[str, Any]]  # pyright: ignore[reportIncompatibleVariableOverride]

    @property
    def ConnectionPool(self) -> Any: ...
    @property
    def client(self) -> Any: ...
    def __init__(
        self,
        host: str | None = None,
        port: int | None = None,
        db: int | None = None,
        password: str | None = None,
        max_connections: int | None = None,
        url: str | None = None,
        connection_pool: Any | None = None,
        **kwargs: Any,
    ) -> None: ...
    def ensure(
        self, fun: Callable[..., Any], args: tuple[Any, ...], **policy: Any
    ) -> Any: ...
    def on_connection_error(
        self, max_retries: int, exc: Exception, intervals: Iterable[float], retries: int
    ) -> None: ...
    @override
    def add_to_chord(self, group_id: str, result: Any) -> None: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
    @override
    def on_chord_part_return(
        self,
        request: Any,
        state: str,
        result: Any,
        propagate: bool | None = None,
        **kwargs: Any,
    ) -> None: ...

class SentinelBackend(RedisBackend):
    sentinel: ModuleType  # redis.sentinel module
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...



from collections.abc import Hashable
from typing import Any

from typing_extensions import override

class EqualityDict(dict[Any, Any]): ...

class HashedSeq(list[Any]):
    __slots__ = "hashvalue"  # noqa: PLC0205  # must match runtime
    hashvalue: int

    def __init__(self, *seq: Any) -> None: ...
    @override
    def __hash__(self) -> int: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleVariableOverride]

def eqhash(o: Any) -> Hashable: ...



from io import FileIO, TextIOWrapper
from pathlib import Path
from typing import Any, NamedTuple

from kombu.transport.virtual import Channel as VirtualChannel
from kombu.transport.virtual import Transport as VirtualTransport
from kombu.transport.virtual.base import BrokerState
from kombu.utils.objects import cached_property
from typing_extensions import override

VERSION: tuple[int, int, int]

def lock(file: FileIO | TextIOWrapper, flags: int) -> None: ...
def unlock(file: FileIO | TextIOWrapper) -> None: ...

class exchange_queue_t(NamedTuple):
    routing_key: str
    pattern: str
    queue: str

class Channel(VirtualChannel):
    supports_fanout: bool
    data_folder_in: Any
    data_folder_out: Any
    processed_folder: Any

    def __init__(self, connection: Any, **kwargs: Any) -> None: ...
    @property
    def control_folder(self) -> Path: ...
    @cached_property
    def store_processed(self) -> bool: ...
    @property
    def transport_options(self) -> dict[str, Any]: ...
    @override
    def get_table(self, exchange: str) -> list[exchange_queue_t]: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    @override
    def _get(self, queue: str) -> dict[str, Any]: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    @override
    def _put(self, queue: str, payload: dict[str, Any], **kwargs: Any) -> None: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    def _put_fanout(
        self, exchange: str, payload: dict[str, Any], routing_key: str, **kwargs: Any
    ) -> None: ...
    def _queue_bind(
        self, exchange: str, routing_key: str, pattern: str | None, queue: str
    ) -> None: ...
    @override
    def _size(self, queue: str) -> int: ...
    @override
    def _purge(self, queue: str) -> int: ...

_Channel = Channel

class Transport(VirtualTransport):
    Channel: type[_Channel]  # pyright: ignore[reportIncompatibleVariableOverride]
    driver_type: str
    driver_name: str
    global_state: BrokerState



from typing import Any

from celery.app.base import Celery

class Events:
    # Class-level configuration
    dispatcher_cls: str
    receiver_cls: str
    state_cls: str

    app: Celery | None

    def __init__(self, app: Celery | None = ...) -> None: ...
    @property
    def Dispatcher(self) -> type[Any]: ...
    @property
    def Receiver(self) -> type[Any]: ...
    @property
    def State(self) -> type[Any]: ...
    def default_dispatcher(
        self,
        hostname: str | None = None,
        enabled: bool = True,
        buffer_while_offline: bool = False,
    ) -> Any: ...



from abc import ABCMeta
from collections import UserList
from collections.abc import Callable, Sequence
from signal import Signals
from typing import Any, TypeAlias

from typing_extensions import override

__all__ = ("Cluster", "Node")

class Cluster(UserList[Node], metaclass=ABCMeta):
    name: str
    nodes: list[Node]

    def __init__(
        self,
        nodes: list[str],
        cmd: str | None = None,
        env: dict[str, str] | None = None,
        on_stopping_preamble: Callable[..., Any] | None = None,
        on_send_signal: Callable[..., Any] | None = None,
        on_still_waiting_for: Callable[..., Any] | None = None,
        on_still_waiting_progress: Callable[..., Any] | None = None,
        on_still_waiting_end: Callable[..., Any] | None = None,
        on_node_start: Callable[..., Any] | None = None,
        on_node_restart: Callable[..., Any] | None = None,
        on_node_shutdown_ok: Callable[..., Any] | None = None,
        on_node_status: Callable[..., Any] | None = None,
        on_node_signal: Callable[..., Any] | None = None,
        on_node_signal_dead: Callable[..., Any] | None = None,
        on_node_down: Callable[..., Any] | None = None,
        on_child_spawn: Callable[..., Any] | None = None,
        on_child_signalled: Callable[..., Any] | None = None,
        on_child_failure: Callable[..., Any] | None = None,
    ) -> None: ...
    def start(self) -> None: ...
    def stop(
        self,
        retry: int | None = None,
        callback: Callable[..., Any] | None = None,
        sig: Signals = ...,
    ) -> None: ...
    def stopwait(
        self,
        retry: int = 2,
        callback: Callable[..., Any] | None = None,
        sig: Signals = ...,
    ) -> None: ...
    def restart(self, sig: Signals = ...) -> None: ...
    def kill(self) -> None: ...
    @property
    @override
    def data(self) -> list[Node]: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleVariableOverride]  # override UserList.data
    def find(self, name: str) -> Node | None: ...
    def getpids(self, on_down: Callable[..., Any] | None = None) -> dict[str, int]: ...
    def send_all(self, sig: Signals) -> None: ...
    def shutdown_nodes(
        self,
        nodes: Sequence[Node],
        sig: Signals = ...,
        retry: int | None = None,
    ) -> None: ...
    def start_node(self, node: Node) -> int: ...

class Node:
    name: str
    cmd: str | None
    append: str | None
    options: dict[str, Any]
    extra_args: Sequence[str]
    pid: int | None

    def __init__(
        self,
        name: str,
        cmd: str | None = None,
        append: str | None = None,
        options: dict[str, Any] | None = None,
        extra_args: Sequence[str] | None = None,
    ) -> None: ...
    def alive(self) -> bool: ...
    def send(
        self, sig: int, on_error: Callable[[Exception], Any] | None = None
    ) -> bool: ...
    def start(self, env: dict[str, str] | None = None, **kwargs: Any) -> int: ...
    @classmethod
    def from_kwargs(cls, name: str, **kwargs: Any) -> Node: ...
    def getopt(self, option: str) -> str | None: ...
    def handle_process_exit(
        self,
        retcode: int,
        on_signalled: Callable[[int], Any] | None = None,
        on_failure: Callable[[int], Any] | None = None,
    ) -> None: ...
    def prepare_argv(self, argv: list[str], path: str) -> list[str]: ...
    @property
    def argv_with_executable(self) -> list[str]: ...
    @property
    def executable(self) -> str: ...
    @property
    def logfile(self) -> str | None: ...
    @property
    def pidfile(self) -> str | None: ...

# Type alias for use inside MultiParser where the class attribute shadows the module-level class
_Node: TypeAlias = Node

class MultiParser:
    Node: type  # type[Node] at runtime
    cmd: str
    append: str
    prefix: str
    suffix: str
    range_prefix: str

    def __init__(
        self,
        cmd: str = "celery worker",
        append: str = "",
        prefix: str = "",
        suffix: str = "",
        range_prefix: str = "celery",
    ) -> None: ...
    def parse(self, p: list[str]) -> tuple[list[_Node], list[str]]: ...

class NamespacedOptionParser:
    args: list[str]
    options: dict[str, dict[str, Any]]
    values: list[str]
    passthrough: str

    def __init__(self, args: list[str]) -> None: ...
    def add_option(
        self, name: str, value: str, short: bool = False, ns: str | None = None
    ) -> None: ...
    def optmerge(
        self, ns: str, defaults: dict[str, Any] | None = None
    ) -> dict[str, Any]: ...
    def parse(self) -> tuple[dict[str, dict[str, Any]], list[str]]: ...
    def process_long_opt(self, arg: str, value: str | None = None) -> None: ...
    def process_short_opt(self, arg: str, value: str | None = None) -> None: ...



from typing import Any

from kombu.exceptions import MessageStateError

__all__ = ("Message",)

ACK_STATES: set[str]

class Message:
    MessageStateError: type[MessageStateError]
    errors: list[Any] | None
    body: str | None
    content_encoding: str | None
    headers: dict[str, str]
    properties: dict[str, str]
    delivery_tag: str | None
    delivery_info: dict[Any, Any]
    content_type: str | None
    accept: Any | None
    channel: Any | None
    def __init__(
        self,
        body: str | None = ...,
        delivery_tag: str | None = ...,
        content_type: str | None = ...,
        content_encoding: str | None = ...,
        delivery_info: Any | None = ...,
        properties: Any | None = ...,
        headers: dict[str, str] | None = ...,
        postencode: Any | None = ...,
        accept: Any | None = ...,
        channel: Any | None = ...,
        **kwargs: object,
    ) -> None: ...
    def ack(self, multiple: bool = ...) -> None: ...
    def ack_log_error(self, logger: Any, errors: Any, multiple: bool = ...) -> None: ...
    def reject_log_error(
        self, logger: Any, errors: Any, requeue: bool = ...
    ) -> None: ...
    def reject(self, requeue: bool = ...) -> None: ...
    def requeue(self) -> None: ...
    def decode(self) -> Any: ...
    def _decode(self) -> Any: ...
    def _reraise_error(self, callback: Any | None = ...) -> None: ...
    @property
    def acknowledged(self) -> bool: ...
    @property
    def payload(self) -> Any: ...



from collections.abc import Iterable
from typing import Any

STANDARD_EXCHANGE_TYPES: dict[str, type[ExchangeType]]

class ExchangeType:
    type: str | None

    def __init__(self, channel: Any) -> None: ...
    def lookup(
        self,
        table: list[tuple[str, str, str]],
        exchange: str,
        routing_key: str,
        default: str | None,
    ) -> Iterable[str]: ...
    def prepare_bind(
        self,
        queue: str,
        exchange: str,
        routing_key: str,
        arguments: dict[str, Any] | None,
    ) -> tuple[str, str, str, dict[str, Any] | None]: ...
    def equivalent(
        self,
        prev: dict[str, Any],
        exchange: str,
        type: str,
        durable: bool,
        auto_delete: bool,
        arguments: dict[str, Any] | None,
    ) -> bool: ...

class DirectExchange(ExchangeType):
    type: str  # pyright: ignore[reportIncompatibleVariableOverride]

    def deliver(
        self,
        message: Any,
        exchange: str,
        routing_key: str,
        **kwargs: Any,
    ) -> None: ...

class TopicExchange(ExchangeType):
    type: str  # pyright: ignore[reportIncompatibleVariableOverride]
    wildcards: dict[str, str]

    def key_to_pattern(self, rkey: str) -> Any: ...
    def deliver(
        self,
        message: Any,
        exchange: str,
        routing_key: str,
        **kwargs: Any,
    ) -> None: ...

class FanoutExchange(ExchangeType):
    type: str  # pyright: ignore[reportIncompatibleVariableOverride]

    def deliver(
        self,
        message: Any,
        exchange: str,
        routing_key: str,
        **kwargs: Any,
    ) -> None: ...



import dataclasses
from concurrent.futures import ThreadPoolExecutor
from logging import Logger
from typing import Any

from google.cloud.monitoring_v3 import MetricServiceClient
from google.cloud.pubsub_v1 import (  # type: ignore[import-untyped]  # pyright: ignore[reportMissingTypeStubs]
    PublisherClient,
    SubscriberClient,
)
from kombu.transport import virtual
from kombu.utils.objects import cached_property
from typing_extensions import override

logger: Logger

PUNCTUATIONS_TO_REPLACE: set[str]
CHARS_REPLACE_TABLE: dict[int, int]

class UnackedIds:
    def __init__(self) -> None: ...
    def append(self, val: Any) -> None: ...
    def extend(self, vals: list[Any]) -> None: ...
    def pop(self, index: int = ...) -> Any: ...
    def remove(self, val: Any) -> None: ...
    def __len__(self) -> int: ...
    def __getitem__(self, item: int) -> Any: ...

class AtomicCounter:
    def __init__(self, initial: int = ...) -> None: ...
    def inc(self, n: int = ...) -> int: ...
    def dec(self, n: int = ...) -> int: ...
    def get(self) -> int: ...

@dataclasses.dataclass
class QueueDescriptor:
    name: str
    topic_path: str
    subscription_id: str
    subscription_path: str
    unacked_ids: UnackedIds = ...

class Channel(virtual.Channel):
    supports_fanout: bool
    do_restore: bool
    default_wait_time_seconds: int
    default_ack_deadline_seconds: int
    default_expiration_seconds: int
    default_retry_timeout_seconds: int
    default_bulk_max_messages: int

    pool: ThreadPoolExecutor
    project_id: str

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def entity_name(self, name: str, table: dict[int, int] = ...) -> str: ...
    @override
    def basic_ack(self, delivery_tag: str, multiple: bool = ...) -> None: ...
    @override
    def after_reply_message_received(self, queue: str) -> None: ...
    @override
    def close(self) -> None: ...
    @cached_property
    def subscriber(self) -> SubscriberClient: ...
    @cached_property
    def publisher(self) -> PublisherClient: ...
    @cached_property
    def monitor(self) -> MetricServiceClient: ...
    @property
    def conninfo(self) -> Any: ...
    @property
    def transport_options(self) -> dict[str, Any]: ...
    @cached_property
    def wait_time_seconds(self) -> int: ...
    @cached_property
    def retry_timeout_seconds(self) -> int: ...
    @cached_property
    def ack_deadline_seconds(self) -> int: ...
    @cached_property
    def queue_name_prefix(self) -> str: ...
    @cached_property
    def expiration_seconds(self) -> int: ...
    @cached_property
    def bulk_max_messages(self) -> int: ...

_Channel = Channel

class Transport(virtual.Transport):
    Channel: type[_Channel]  # pyright: ignore[reportIncompatibleVariableOverride]
    can_parse_url: bool
    polling_interval: float  # pyright: ignore[reportIncompatibleVariableOverride]
    connection_errors: tuple[type[BaseException], ...]
    channel_errors: tuple[type[BaseException], ...]
    driver_type: str
    driver_name: str
    implements: Any

    def __init__(self, client: Any, **kwargs: Any) -> None: ...
    @override
    def driver_version(self) -> str: ...
    @staticmethod
    def parse_uri(uri: str) -> str: ...
    @classmethod
    @override
    def as_uri(cls, uri: str, include_password: bool = ..., mask: str = ...) -> str: ...  # pyright: ignore[reportIncompatibleMethodOverride]
    @override
    def drain_events(self, connection: Any, timeout: float | None = ...) -> None: ...  # type: ignore[override]



from multiprocessing.context import BaseContext
from types import TracebackType
from typing import Any

from typing_extensions import Self

__all__ = ["BaseManager", "BaseProxy", "SyncManager", "Token"]

class Token:
    typeid: str | None
    address: tuple[str, int] | str | None
    id: str | None
    def __init__(
        self,
        typeid: str | None,
        address: tuple[str, int] | str | None,
        id: str | None,
    ) -> None: ...

class BaseProxy:
    def __init__(
        self,
        token: Any,
        serializer: str,
        manager: Any = ...,
        authkey: str | bytes | None = ...,
        exposed: Any = ...,
        incref: bool = ...,
    ) -> None: ...
    def __deepcopy__(self, memo: dict[int, Any]) -> BaseProxy: ...

class BaseManager:
    def __init__(
        self,
        address: str | tuple[str, int] | None = ...,
        authkey: bytes | None = ...,
        serializer: str = ...,
        ctx: BaseContext | None = ...,
    ) -> None: ...
    def start(
        self, initializer: Any = ..., initargs: tuple[Any, ...] = ...
    ) -> None: ...
    def get_server(self) -> Any: ...
    def connect(self) -> None: ...
    def join(self, timeout: float | None = ...) -> None: ...
    @classmethod
    def register(
        cls,
        typeid: str,
        callable: Any = ...,
        proxytype: type[Any] | None = ...,
        exposed: Any = ...,
        method_to_typeid: dict[str, str] | None = ...,
        create_method: bool = ...,
    ) -> None: ...
    @property
    def address(self) -> tuple[str, int] | str | None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...

class SyncManager(BaseManager):
    def Barrier(self, *args: Any, **kwds: Any) -> Any: ...
    def BoundedSemaphore(self, *args: Any, **kwds: Any) -> Any: ...
    def Condition(self, *args: Any, **kwds: Any) -> Any: ...
    def Event(self, *args: Any, **kwds: Any) -> Any: ...
    def JoinableQueue(self, *args: Any, **kwds: Any) -> Any: ...
    def Lock(self, *args: Any, **kwds: Any) -> Any: ...
    def Namespace(self, *args: Any, **kwds: Any) -> Any: ...
    def Pool(self, *args: Any, **kwds: Any) -> Any: ...
    def Queue(self, *args: Any, **kwds: Any) -> Any: ...
    def RLock(self, *args: Any, **kwds: Any) -> Any: ...
    def Semaphore(self, *args: Any, **kwds: Any) -> Any: ...
    def Value(self, *args: Any, **kwds: Any) -> Any: ...
    def Array(self, *args: Any, **kwds: Any) -> Any: ...
    def dict(self, *args: Any, **kwds: Any) -> Any: ...
    def list(self, *args: Any, **kwds: Any) -> Any: ...



from typing import Any

from celery.app.base import Celery
from celery.events.state import State
from typing_extensions import Self

__all__ = ("Polaroid", "evcam")

class Polaroid:
    app: Celery
    state: State
    freq: float
    timer: Any

    def __init__(
        self,
        state: State,
        freq: float = 1.0,
        maxrate: str | None = None,
        cleanup_freq: float = 3600.0,
        timer: Any | None = None,
        app: Celery | None = None,
    ) -> None: ...
    cleanup_signal: Any
    clear_after: bool
    shutter_signal: Any

    def install(self) -> None: ...
    def on_cleanup(self) -> None: ...
    def on_shutter(self, state: State) -> None: ...
    def shutter(self) -> None: ...
    def capture(self) -> None: ...
    def cancel(self) -> None: ...
    def cleanup(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *args: object) -> None: ...

def evcam(
    camera: str,
    freq: float = 1.0,
    maxrate: str | None = None,
    loglevel: int = 0,
    logfile: str | None = None,
    pidfile: str | None = None,
    timer: Any | None = None,
    app: Celery | None = None,
    **kwargs: Any,
) -> None: ...



from collections.abc import Callable, Generator
from typing import Any

from kombu.asynchronous.timer import Timer
from kombu.utils.objects import cached_property

__all__ = ("Hub", "get_event_loop", "set_event_loop")

READ: int
WRITE: int
ERR: int

def _dummy_context(*args: Any) -> Generator[None, None, None]: ...
def _raise_stop_error() -> None: ...

class Hub:
    READ: int
    WRITE: int
    ERR: int

    timer: Timer
    readers: dict[int, Callable[..., Any]]
    writers: dict[int, Callable[..., Any]]
    on_tick: list[Callable[..., Any]]
    on_close: list[Callable[..., Any]] | None

    def __init__(self, timer: Timer | None = ...) -> None: ...
    def add(
        self,
        fd: int | Any,
        callback: Callable[..., Any],
        flags: int,
        args: tuple[Any, ...] = ...,
        consolidate: bool = ...,
    ) -> None: ...
    def add_reader(
        self, fds: int | Any, callback: Callable[..., Any], *args: Any
    ) -> None: ...
    def add_writer(
        self, fds: int | Any, callback: Callable[..., Any], *args: Any
    ) -> None: ...
    def remove(self, fd: int | Any) -> None: ...
    def remove_reader(self, fd: int | Any) -> None: ...
    def remove_writer(self, fd: int | Any) -> None: ...
    def run_forever(self) -> None: ...
    def run_once(self) -> None: ...
    def call_soon(self, callback: Callable[..., Any], *args: Any) -> Any: ...
    def call_later(
        self, delay: float, callback: Callable[..., Any], *args: Any
    ) -> Any: ...
    def call_at(self, when: float, callback: Callable[..., Any], *args: Any) -> Any: ...
    def call_repeatedly(
        self, delay: float, callback: Callable[..., Any], *args: Any
    ) -> Any: ...
    def close(self, *args: Any) -> None: ...
    def stop(self) -> None: ...
    def reset(self) -> None: ...
    def fire_timers(
        self,
        min_delay: float = ...,
        max_delay: float = ...,
        max_timers: int = ...,
        propagate: tuple[type[Exception], ...] = ...,
    ) -> float: ...
    def create_loop(
        self,
        generator: type[Generator[Any, Any, Any]] = ...,
        sleep: Callable[[float], None] = ...,
        min: Callable[..., Any] = ...,
        next: Callable[..., Any] = ...,
        Empty: type[Exception] = ...,
        StopIteration: type[BaseException] = ...,
        KeyError: type[Exception] = ...,
        READ: int = ...,
        WRITE: int = ...,
        ERR: int = ...,
    ) -> Generator[None, None, None]: ...
    def on_callback_error(
        self, callback: Callable[..., Any], exc: BaseException
    ) -> None: ...
    def repr_active(self) -> str: ...
    def repr_events(self, events: list[tuple[int, int]]) -> str: ...
    @property
    def loop(self) -> Generator[None, None, None]: ...
    @property
    def poller(self) -> Any: ...
    def _create_poller(self) -> Any: ...
    def _close_poller(self) -> None: ...
    def _pop_ready(self) -> Callable[..., Any] | None: ...
    def _unregister(self, fd: int | Any) -> None: ...
    def _remove_from_loop(self, fd: int | Any) -> None: ...
    def _discard(self, fd: int | Any) -> None: ...
    @cached_property
    def scheduler(self) -> Timer: ...

def get_event_loop() -> Hub | None: ...
def set_event_loop(loop: Hub | None) -> Hub | None: ...



from collections.abc import Iterator
from datetime import datetime
from typing import Any

from kombu.transport.virtual import Channel as VirtualChannel
from kombu.transport.virtual import Transport as VirtualTransport
from kombu.utils.objects import cached_property
from pymongo.collection import Collection
from pymongo.cursor import Cursor
from pymongo.database import Database
from typing_extensions import override

E_SERVER_VERSION: str
E_NO_TTL_INDEXES: str

class BroadcastCursor:
    def __init__(self, cursor: Cursor[Any]) -> None: ...
    def get_size(self) -> int: ...
    def close(self) -> None: ...
    def purge(self, rewind: bool = ...) -> None: ...
    def __iter__(self) -> Iterator[Any]: ...
    def __next__(self) -> Any: ...
    def next(self) -> Any: ...

class Channel(VirtualChannel):
    supports_fanout: bool

    ssl: bool
    ttl: bool
    connect_timeout: int | None
    capped_queue_size: int
    calc_queue_size: bool

    default_hostname: str
    default_port: int
    default_database: str

    messages_collection: str
    routing_collection: str
    broadcast_collection: str
    queues_collection: str

    from_transport_options: tuple[str, ...]

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @override
    def get_table(self, exchange: str) -> frozenset[tuple[str, str, str]]: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    @override
    def queue_delete(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self, queue: str, **kwargs: Any
    ) -> int | None: ...
    @override
    def prepare_queue_arguments(
        self, arguments: dict[str, Any] | None, **kwargs: Any
    ) -> dict[str, Any]: ...
    def get_now(self) -> datetime: ...
    @cached_property
    def client(self) -> Database[Any]: ...
    @cached_property
    def messages(self) -> Collection[Any]: ...
    @cached_property
    def routing(self) -> Collection[Any]: ...
    @cached_property
    def broadcast(self) -> Collection[Any]: ...
    def _prepare_client_options(self, options: dict[str, Any]) -> dict[str, Any]: ...
    def _parse_uri(
        self, scheme: str = ...
    ) -> tuple[str, str, str | None, str | None]: ...
    def _open(self, scheme: str = ...) -> Database[Any]: ...
    @override
    def _new_queue(self, queue: str, **kwargs: Any) -> None: ...
    @override
    def _get(self, queue: str) -> Any: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    def _create_broadcast_cursor(
        self, exchange: str, routing_key: str, pattern: str, queue: str
    ) -> BroadcastCursor: ...
    def _queue_bind(
        self, exchange: str, routing_key: str, pattern: str, queue: str
    ) -> None: ...
    def _get_broadcast_cursor(self, queue: str) -> BroadcastCursor | None: ...
    @override
    def _put(self, queue: str, message: dict[str, Any], **kwargs: Any) -> None: ...
    def _put_fanout(
        self, exchange: str, message: dict[str, Any], routing_key: str, **kwargs: Any
    ) -> None: ...
    @override
    def _size(self, queue: str) -> int: ...
    @override
    def _purge(self, queue: str) -> int: ...
    def _create_broadcast(self, database: Any) -> None: ...
    def _ensure_indexes(self, database: Any) -> None: ...
    def _get_queue_expire(self, queue: str, argument: str) -> datetime | None: ...
    def _update_queues_expire(self, queue: str) -> None: ...
    def _get_message_expire(self, message: Any) -> datetime | None: ...
    @cached_property
    def queues(self) -> Collection[Any]: ...

_Channel = Channel

class Transport(VirtualTransport):
    Channel: type[_Channel]  # pyright: ignore[reportIncompatibleVariableOverride]

    can_parse_url: bool
    polling_interval: int  # pyright: ignore[reportIncompatibleVariableOverride]
    default_port: int  # pyright: ignore[reportIncompatibleVariableOverride]
    connection_errors: tuple[type[Exception], ...]  # pyright: ignore[reportIncompatibleVariableOverride]
    channel_errors: tuple[type[Exception], ...]  # pyright: ignore[reportIncompatibleVariableOverride]
    driver_type: str
    driver_name: str
    implements: Any

    @override
    def driver_version(self) -> str: ...
    @override
    def as_uri(
        self, uri: str, include_password: bool = ..., mask: str = ...
    ) -> str: ...



from collections.abc import Callable
from functools import partial as partial
from types import TracebackType
from typing import Any, TypeVar

_F = TypeVar("_F", bound=Callable[..., Any])

WRAPPER_ASSIGNMENTS: tuple[str, ...]
WRAPPER_UPDATES: tuple[str, ...]

def update_wrapper(
    wrapper: _F,
    wrapped: Callable[..., Any],
    assigned: tuple[str, ...] = ...,
    updated: tuple[str, ...] = ...,
) -> _F: ...
def wraps(
    wrapped: Callable[..., Any],
    assigned: tuple[str, ...] = ...,
    updated: tuple[str, ...] = ...,
) -> Callable[[_F], _F]: ...
def reraise(
    tp: type[BaseException], value: BaseException, tb: TracebackType | None = ...
) -> None: ...

__all__ = ["update_wrapper", "wraps"]



import shelve
from collections.abc import Callable
from datetime import datetime
from threading import Thread
from typing import Any, NamedTuple

import kombu
from celery.app.base import Celery
from celery.schedules import BaseSchedule
from typing_extensions import override

__all__ = (
    "EmbeddedService",
    "PersistentScheduler",
    "ScheduleEntry",
    "Scheduler",
    "SchedulingError",
    "Service",
)

class SchedulingError(Exception): ...

class BeatLazyFunc:
    def __init__(self, func: Callable[..., Any], *args: Any, **kwargs: Any) -> None: ...
    def __call__(self) -> Any: ...
    def delay(self) -> Any: ...

class ScheduleEntry:
    total_run_count: int
    name: str | None
    task: str | None
    last_run_at: datetime | None
    schedule: BaseSchedule | None
    args: tuple[Any, ...] | None
    kwargs: dict[str, Any] | None
    options: dict[str, Any] | None
    relative: bool
    app: Celery | None

    def __init__(
        self,
        name: str | None = None,
        task: str | None = None,
        last_run_at: datetime | None = None,
        total_run_count: int | None = None,
        schedule: BaseSchedule | None = None,
        args: tuple[Any, ...] = (),
        kwargs: dict[str, Any] | None = None,
        options: dict[str, Any] | None = None,
        relative: bool = False,
        app: Celery | None = None,
    ) -> None: ...
    def __ge__(self, other: ScheduleEntry) -> bool: ...
    def __gt__(self, other: ScheduleEntry) -> bool: ...
    def __le__(self, other: ScheduleEntry) -> bool: ...
    def __lt__(self, other: ScheduleEntry) -> bool: ...
    def __iter__(self) -> ScheduleEntry: ...
    def __next__(self, last_run_at: datetime | None = None) -> ScheduleEntry: ...
    def default_now(self) -> datetime: ...
    def editable_fields_equal(self, other: ScheduleEntry) -> bool: ...
    def is_due(self) -> tuple[bool, float]: ...
    def next(self, last_run_at: datetime | None = None) -> ScheduleEntry: ...
    def update(self, other: ScheduleEntry) -> None: ...

class event_t(NamedTuple):
    time: float
    priority: int
    entry: ScheduleEntry

class Scheduler:
    Entry: type[ScheduleEntry]
    app: Celery
    schedule: dict[str, ScheduleEntry]
    max_interval: int | None
    lazy: bool

    def __init__(
        self,
        app: Celery,
        schedule: dict[str, ScheduleEntry] | None = None,
        max_interval: int | None = None,
        Producer: type[kombu.Producer] | None = None,
        lazy: bool = False,
        sync_every_tasks: int | None = None,
        **kwargs: Any,
    ) -> None: ...
    def setup_schedule(self) -> None: ...
    def sync(self) -> None: ...
    def close(self) -> None: ...
    def add(self, **kwargs: Any) -> ScheduleEntry: ...
    def adjust(self, n: int, drift: float = -0.01) -> float: ...
    def apply_async(
        self,
        entry: ScheduleEntry,
        producer: kombu.Producer | None = None,
        advance: bool = True,
        **kwargs: Any,
    ) -> Any: ...
    def apply_entry(
        self, entry: ScheduleEntry, producer: kombu.Producer | None = None
    ) -> Any: ...
    @property
    def connection(self) -> kombu.Connection: ...
    def get_schedule(self) -> dict[str, ScheduleEntry]: ...
    @property
    def info(self) -> str: ...
    def install_default_entries(self, data: dict[str, Any]) -> None: ...
    def is_due(self, entry: ScheduleEntry) -> tuple[bool, float]: ...
    @property
    def logger(self) -> Any: ...
    def merge_inplace(self, b: dict[str, ScheduleEntry]) -> None: ...
    def populate_heap(
        self, event_t: type[event_t] = ..., heapify: Callable[[list[Any]], None] = ...
    ) -> None: ...
    @property
    def producer(self) -> kombu.Producer: ...
    def reserve(self, entry: ScheduleEntry) -> event_t: ...
    def schedules_equal(
        self,
        old_schedules: dict[str, ScheduleEntry],
        new_schedules: dict[str, ScheduleEntry],
    ) -> bool: ...
    def send_task(self, *args: Any, **kwargs: Any) -> Any: ...
    def set_schedule(self, schedule: dict[str, ScheduleEntry]) -> None: ...
    def should_sync(self) -> bool: ...
    @property
    def sync_every(self) -> int: ...
    @property
    def sync_every_tasks(self) -> int | None: ...
    def tick(
        self,
        event_t: type[event_t] = ...,
        min: Callable[..., Any] = ...,
        heappop: Callable[..., Any] = ...,
        heappush: Callable[..., Any] = ...,
    ) -> float: ...
    def update_from_dict(self, dict_: dict[str, Any]) -> None: ...

class PersistentScheduler(Scheduler):
    persistence: type[shelve.Shelf[Any]]
    known_suffixes: tuple[str, ...]
    @override
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @override
    def setup_schedule(self) -> None: ...
    @override
    def sync(self) -> None: ...
    @override
    def close(self) -> None: ...
    @override
    def get_schedule(self) -> dict[str, ScheduleEntry]: ...
    @override
    def set_schedule(self, schedule: dict[str, ScheduleEntry]) -> None: ...
    @property
    @override
    def info(self) -> str: ...

class Service:
    scheduler_cls: type[PersistentScheduler]
    app: Celery
    max_interval: int | None

    def __init__(
        self,
        app: Celery,
        max_interval: int | None = None,
        schedule_filename: str | None = None,
        scheduler_cls: type[Scheduler] | None = None,
    ) -> None: ...
    def start(self, embedded_process: bool = ...) -> None: ...
    def sync(self) -> None: ...
    def stop(self, wait: bool = ...) -> None: ...
    def get_scheduler(
        self, lazy: bool = ..., extension_namespace: str = ...
    ) -> Scheduler: ...
    @property
    def scheduler(self) -> Scheduler: ...

class _Threaded(Thread):
    daemon: bool
    name: str
    app: Celery

    def __init__(self, app: Celery, **kwargs: Any) -> None: ...
    @override
    def run(self) -> None: ...
    def stop(self) -> None: ...

class _Process:
    name: str
    app: Celery

    def __init__(self, app: Celery, **kwargs: Any) -> None: ...
    def run(self) -> None: ...
    def stop(self) -> None: ...

def EmbeddedService(
    app: Celery, max_interval: int | None = None, **kwargs: Any
) -> _Threaded | _Process: ...



from collections.abc import Callable
from logging import Logger
from typing import Any

__all__ = (
    "TraceInfo",
    "build_tracer",
    "reset_worker_optimizations",
    "setup_worker_optimizations",
    "trace_task",
)

from celery.app.base import Celery
from celery.app.task import Task

class TraceInfo:
    state: str
    retval: Any

    def __init__(self, state: str, retval: Any = None) -> None: ...
    def handle_error_state(
        self,
        task: Task[..., Any],
        req: Any,
        eager: bool = False,
        call_errbacks: bool = True,
    ) -> None: ...
    def handle_failure(
        self,
        task: Task[..., Any],
        req: Any,
        store_errors: bool = True,
        call_errbacks: bool = True,
    ) -> None: ...
    def handle_ignore(self, task: Task[..., Any], req: Any) -> None: ...
    def handle_reject(self, task: Task[..., Any], req: Any, **kwargs: Any) -> None: ...
    def handle_retry(
        self,
        task: Task[..., Any],
        req: Any,
        store_errors: bool = True,
        **kwargs: Any,
    ) -> None: ...

def build_tracer(
    name: str,
    task: Task[..., Any],
    loader: Any | None = None,
    hostname: str | None = None,
    store_errors: bool = True,
    Info: type = ...,
    eager: bool = False,
    propagate: bool = False,
    app: Celery | None = None,
    monotonic: Callable[[], float] = ...,
    trace_ok_t: type = ...,
    IGNORE_STATES: frozenset[str] = ...,
) -> Callable[..., Any]: ...
def trace_task(
    task: Task[..., Any],
    uuid: str,
    args: tuple[Any, ...],
    kwargs: dict[str, Any],
    request: Any | None = None,
    **opts: Any,
) -> TraceInfo: ...
def trace_task_ret(
    name: str,
    uuid: str,
    request: Any,
    body: Any,
    content_type: str,
    content_encoding: str,
    loads: Callable[..., Any] = ...,
    app: Any = None,
    **extra_request: Any,
) -> TraceInfo: ...
def fast_trace_task(
    task: Task[..., Any],
    uuid: str,
    request: Any,
    body: Any,
    content_type: str,
    content_encoding: str,
    loads: Callable[..., Any] = ...,
    _loc: Any = None,
    hostname: str | None = None,
    **extra_request: Any,
) -> TraceInfo: ...
def setup_worker_optimizations(app: Celery, hostname: str | None = None) -> None: ...
def reset_worker_optimizations(app: Any = ...) -> None: ...

logger: Logger



from collections.abc import Iterable
from logging import Formatter, Logger, LogRecord, _SysExcInfoType
from typing import ClassVar

from celery.utils.term import colored
from typing_extensions import override

__all__ = (
    "LOG_LEVELS",
    "ColorFormatter",
    "LoggingProxy",
    "base_logger",
    "get_logger",
    "get_multiprocessing_logger",
    "get_task_logger",
    "in_sighandler",
    "mlevel",
    "reset_multiprocessing_logger",
    "set_in_sighandler",
)

LOG_LEVELS: dict[str, int]
base_logger: Logger
task_logger: Logger
worker_logger: Logger

def get_logger(name: str) -> Logger: ...
def get_multiprocessing_logger() -> Logger | None: ...
def get_task_logger(name: str) -> Logger: ...
def in_sighandler() -> bool: ...
def mlevel(level: int | str) -> int: ...
def reset_multiprocessing_logger() -> None: ...
def set_in_sighandler(value: bool) -> None: ...

class LoggingProxy:
    loglevel: int
    mode: str
    name: str | None
    closed: bool

    def __init__(self, logger: Logger, loglevel: int | None = None) -> None: ...
    def close(self) -> None: ...
    def flush(self) -> None: ...
    def isatty(self) -> bool: ...
    def write(self, data: str) -> None: ...
    def writelines(self, sequence: Iterable[str]) -> None: ...

class ColorFormatter(Formatter):
    use_color: bool

    COLORS: ClassVar[dict[str, colored]]
    colors: ClassVar[dict[str, colored]]
    def __init__(self, fmt: str | None = ..., use_color: bool = ...) -> None: ...
    @override
    def formatException(self, ei: _SysExcInfoType) -> str: ...
    @override
    def format(self, record: LogRecord) -> str: ...



from typing import Any

from celery.backends.base import KeyValueStoreBackend

__all__ = ("AzureBlockBlobBackend",)

class AzureBlockBlobBackend(KeyValueStoreBackend):
    def __init__(
        self,
        url: str | None = None,
        container_name: str | None = None,
        connection_string: str | None = None,
        **kwargs: Any,
    ) -> None: ...



from collections.abc import Callable, Container, Generator, Iterable
from typing import Any, TypeAlias, TypeGuard, TypeVar

from kombu.utils.functional import (
    LRUCache,
    dictfilter,
    is_list,
    lazy,
    maybe_evaluate,
    maybe_list,
    memoize,
)
from typing_extensions import Self

__all__ = (
    "LRUCache",
    "chunks",
    "dictfilter",
    "first",
    "firstmethod",
    "fun_accepts_kwargs",
    "head_from_fun",
    "is_list",
    "lazy",
    "mattrgetter",
    "maybe",
    "maybe_evaluate",
    "maybe_list",
    "memoize",
    "mlazy",
    "noop",
    "padlist",
    "regen",
    "uniq",
)

class DummyContext:
    def __enter__(self) -> Self: ...
    def __exit__(self, *exc_info: object) -> None: ...

class mlazy:
    evaluated: bool
    _value: Any

    def evaluate(self) -> Any: ...

def noop(*args: object, **kwargs: object) -> None: ...

_T = TypeVar("_T")

def pass1(arg: _T, *args: object, **kwargs: object) -> _T: ...
def evaluate_promises(it: Iterable[Any]) -> Generator[Any, None, None]: ...
def first(predicate: Callable[[_T], bool] | None, it: Iterable[_T]) -> _T | None: ...
def firstmethod(
    method: Callable[..., Any], on_call: Callable[..., Any] | None = None
) -> Callable[..., Any]: ...
def chunks(it: Iterable[_T], n: int) -> Generator[list[_T], None, None]: ...
def padlist(
    container: Container[_T], size: int, default: _T | None = None
) -> list[_T]: ...
def mattrgetter(*attrs: str) -> Callable[[object], dict[str, Any]]: ...
def uniq(it: Iterable[_T]) -> Generator[_T, None, None]: ...
def lookahead(it: Iterable[_T]) -> Iterable[tuple[_T, _T | None]]: ...
def regen(it: Iterable[_T]) -> list[_T] | tuple[_T]: ...

_FuncType: TypeAlias = Callable[..., Any]

def head_from_fun(fun: _FuncType, bound: bool = False) -> _FuncType: ...
def arity_greater(fun: _FuncType, n: int) -> bool: ...
def fun_takes_argument(
    name: str, fun: _FuncType, position: int | None = None
) -> bool: ...
def fun_accepts_kwargs(fun: _FuncType) -> bool: ...
def maybe(typ: Callable[..., _T], val: object | None) -> _T | None: ...
def seq_concat_item(seq: list[_T] | tuple[_T], item: _T) -> list[_T] | tuple[_T]: ...
def seq_concat_seq(
    a: list[Any] | tuple[Any], b: list[Any] | tuple[Any]
) -> list[Any] | tuple[Any]: ...
def is_numeric_value(value: Any) -> TypeGuard[bool]: ...



from collections.abc import Callable
from typing import Any, Generic, TypeAlias, TypeVar

__all__ = ("PromiseProxy", "Proxy", "maybe_evaluate", "try_import")

from typing_extensions import Self, override

_T = TypeVar("_T")
_R = TypeVar("_R")

_Getter: TypeAlias = Callable[[_T], _R]
_Setter: TypeAlias = Callable[[_T, _R], None]

def maybe_evaluate(obj: Any) -> Any: ...
def try_import(module: str, default: _T = ...) -> Any | _T: ...

class class_property(Generic[_T, _R]):
    __get: _Getter[_T, _R]
    __set: _Setter[_T, _R] | None
    def __init__(
        self,
        getter: _Getter[_T, _R] | None = None,
        setter: _Setter[_T, _R] | None = None,
    ) -> None: ...
    def __get__(self, obj: _T | None, type: type[_T] | None = None) -> _R: ...
    def __set__(self, obj: _T | None, value: _R) -> Self: ...
    def setter(self, setter: _Setter[_T, _R]) -> Self: ...

class Proxy(Generic[_T]):
    def __init__(
        self,
        local: Callable[[], _T],
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        name: str | None = ...,
        __doc__: str | None = ...,
    ) -> None: ...
    def _get_current_object(self) -> _T: ...
    # All the dunder methods that proxy to the wrapped object
    def __abs__(self) -> Any: ...
    def __add__(self, other: Any) -> Any: ...
    def __and__(self, other: Any) -> Any: ...
    def __bool__(self) -> bool: ...
    def __call__(self, *a: Any, **kw: Any) -> Any: ...
    def __coerce__(self, other: Any) -> Any: ...
    def __complex__(self) -> complex: ...
    def __contains__(self, i: Any) -> bool: ...
    def __delitem__(self, key: Any) -> None: ...
    def __divmod__(self, other: Any) -> Any: ...
    def __enter__(self) -> Any: ...
    def __exit__(self, *a: object, **kw: Any) -> Any: ...
    def __float__(self) -> float: ...
    def __floordiv__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> bool: ...
    def __getitem__(self, i: Any) -> Any: ...
    def __gt__(self, other: Any) -> bool: ...
    def __hex__(self) -> str: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __invert__(self) -> Any: ...
    def __iter__(self) -> Any: ...
    def __le__(self, other: Any) -> bool: ...
    def __len__(self) -> int: ...
    def __lt__(self, other: Any) -> bool: ...
    def __mod__(self, other: Any) -> Any: ...
    def __mul__(self, other: Any) -> Any: ...
    @override
    def __ne__(self, other: object) -> bool: ...
    def __neg__(self) -> Any: ...
    def __oct__(self) -> str: ...
    def __or__(self, other: Any) -> Any: ...
    def __pos__(self) -> Any: ...
    def __pow__(self, other: Any) -> Any: ...
    @override
    def __reduce__(self) -> tuple[Any, ...]: ...
    def __rshift__(self, other: Any) -> Any: ...
    def __lshift__(self, other: Any) -> Any: ...
    def __setitem__(self, key: Any, value: Any) -> None: ...
    def __sub__(self, other: Any) -> Any: ...
    def __truediv__(self, other: Any) -> Any: ...
    def __xor__(self, other: Any) -> Any: ...
    @property
    def __name__(self) -> str: ...

class PromiseProxy(Proxy[_T]):
    def __init__(
        self,
        local: Callable[[], _T],
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        name: str | None = ...,
        __doc__: str | None = ...,
    ) -> None: ...
    def __evaluate__(
        self,
        _clean: tuple[str, ...] = ("_Proxy__local", "_Proxy__args", "_Proxy__kwargs"),
    ) -> _T: ...
    def __evaluated__(self) -> bool: ...
    def __maybe_evaluate__(self) -> _T: ...
    @property
    def __pending__(self) -> bool: ...
    def __then__(self, fun: Callable[..., Any], *args: Any, **kwargs: Any) -> Any: ...



from datetime import datetime
from typing import Any, NamedTuple, TypeAlias

import kombu
import kombu.pools
from celery.app.base import Celery
from celery.app.routes import Router as RouterClass
from kombu.transport.base import StdChannel

__all__ = ("AMQP", "Queues", "task_message")

class task_message(NamedTuple):
    headers: dict[str, Any]
    properties: dict[str, Any]
    body: tuple[Any, ...]
    sent_event: dict[str, Any] | None

class Queues(dict[str, kombu.Queue]):
    def __init__(
        self,
        queues: list[kombu.Queue] | dict[str, kombu.Queue] | None = None,
        default_exchange: kombu.Exchange | None = None,
        create_missing: bool = True,
        create_missing_queue_type: str | None = None,
        create_missing_queue_exchange_type: str | None = None,
        autoexchange: kombu.Exchange | None = None,
        max_priority: int | None = None,
        default_routing_key: str | None = None,
    ) -> None: ...
    def __missing__(self, name: str) -> kombu.Queue: ...
    def add(self, queue: kombu.Queue, **kwargs: Any) -> None: ...
    def add_compat(self, name: str, **options: Any) -> kombu.Queue: ...
    @property
    def consume_from(self) -> dict[str, kombu.Queue]: ...
    def deselect(self, exclude: list[str]) -> None: ...
    def format(self, indent: int = 0, indent_first: bool = True) -> str: ...
    def new_missing(self, name: str) -> kombu.Queue: ...
    def select(self, include: list[str]) -> None: ...
    def select_add(self, queue: kombu.Queue, **kwargs: Any) -> None: ...

# Type alias to avoid conflict with AMQP.Queues method
_Queues: TypeAlias = Queues

class AMQP:
    # Class attributes
    BrokerConnection: type[kombu.Connection]
    Connection: type[kombu.Connection]
    Consumer: type[kombu.Consumer]
    Producer: type[kombu.Producer]
    queues_cls: type[_Queues]
    argsrepr_maxsize: int
    kwargsrepr_maxsize: int
    autoexchange: kombu.Exchange | None

    app: Celery

    def __init__(self, app: Celery) -> None: ...
    def TaskConsumer(
        self,
        channel: StdChannel,
        queues: list[kombu.Queue] | None = None,
        accept: list[str] | None = None,
        **kw: Any,
    ) -> kombu.Consumer: ...
    def Queues(
        self,
        queues: list[kombu.Queue] | dict[str, kombu.Queue],
        create_missing: bool | None = None,
        create_missing_queue_type: str | None = None,
        create_missing_queue_exchange_type: str | None = None,
        autoexchange: kombu.Exchange | None = None,
        max_priority: int | None = None,
    ) -> _Queues: ...
    def Router(
        self,
        queues: _Queues | None = None,
        create_missing: bool | None = None,
    ) -> RouterClass: ...
    def flush_routes(self) -> None: ...
    def as_task_v1(
        self,
        task_id: str,
        name: str,
        args: tuple[Any, ...] | None = None,
        kwargs: dict[str, Any] | None = None,
        countdown: float | None = None,
        eta: datetime | None = None,
        group_id: str | None = None,
        group_index: int | None = None,
        expires: float | datetime | None = None,
        retries: int = 0,
        chord: Any | None = None,
        callbacks: list[Any] | None = None,
        errbacks: list[Any] | None = None,
        reply_to: str | None = None,
        time_limit: int | None = None,
        soft_time_limit: int | None = None,
        create_sent_event: bool = False,
        root_id: str | None = None,
        parent_id: str | None = None,
        shadow: str | None = None,
        now: datetime | None = None,
        timezone: Any | None = None,
        **compat_kwargs: Any,
    ) -> task_message: ...
    def as_task_v2(
        self,
        task_id: str,
        name: str,
        args: tuple[Any, ...] | None = None,
        kwargs: dict[str, Any] | None = None,
        countdown: float | None = None,
        eta: datetime | None = None,
        group_id: str | None = None,
        group_index: int | None = None,
        expires: float | datetime | None = None,
        retries: int = 0,
        chord: Any | None = None,
        callbacks: list[Any] | None = None,
        errbacks: list[Any] | None = None,
        reply_to: str | None = None,
        time_limit: int | None = None,
        soft_time_limit: int | None = None,
        create_sent_event: bool = False,
        root_id: str | None = None,
        parent_id: str | None = None,
        shadow: str | None = None,
        chain: Any | None = None,
        now: datetime | None = None,
        timezone: Any | None = None,
        origin: str | None = None,
        ignore_result: bool = False,
        argsrepr: str | None = None,
        kwargsrepr: str | None = None,
        stamped_headers: list[str] | None = None,
        replaced_task_nesting: int = 0,
        **options: Any,
    ) -> task_message: ...
    @property
    def create_task_message(self) -> Any: ...
    @property
    def default_exchange(self) -> kombu.Exchange: ...
    @property
    def default_queue(self) -> kombu.Queue: ...
    @property
    def producer_pool(self) -> kombu.pools.ProducerPool: ...
    @property
    def publisher_pool(self) -> kombu.pools.ProducerPool: ...
    @property
    def queues(self) -> _Queues: ...
    @property
    def router(self) -> RouterClass: ...
    @property
    def routes(self) -> list[dict[str, Any]]: ...
    @property
    def send_task_message(self) -> Any: ...
    @property
    def utc(self) -> bool: ...



from typing import Any

from kombu.asynchronous.http.base import BaseClient
from kombu.asynchronous.hub import Hub
from typing_extensions import override

__all__ = ("CurlClient",)

class CurlClient(BaseClient):
    Curl: type[Any] | None
    max_clients: int
    _multi: Any
    _curls: list[Any]
    _free_list: list[Any]
    _pending: Any
    _fds: dict[int, int]
    _socket_action: Any
    _timeout_check_tref: Any

    def __init__(self, hub: Hub | None = ..., max_clients: int = ...) -> None: ...
    @override
    def close(self) -> None: ...
    @override
    def add_request(self, request: Any) -> Any: ...
    def on_readable(self, fd: int, _pycurl: Any = ...) -> Any: ...
    def on_writable(self, fd: int, _pycurl: Any = ...) -> Any: ...



from collections.abc import Callable, Mapping, Sequence
from typing import Any

from amqp.basic_message import Message as Message
from amqp.channel import Channel as Channel
from amqp.connection import Connection as Connection
from amqp.exceptions import (
    AccessRefused as AccessRefused,
)
from amqp.exceptions import (
    AMQPError as AMQPError,
)
from amqp.exceptions import (
    AMQPNotImplementedError as AMQPNotImplementedError,
)
from amqp.exceptions import (
    ChannelError as ChannelError,
)
from amqp.exceptions import (
    ChannelNotOpen as ChannelNotOpen,
)
from amqp.exceptions import (
    ConnectionError as ConnectionError,
)
from amqp.exceptions import (
    ConnectionForced as ConnectionForced,
)
from amqp.exceptions import (
    ConsumerCancelled as ConsumerCancelled,
)
from amqp.exceptions import (
    ContentTooLarge as ContentTooLarge,
)
from amqp.exceptions import (
    FrameError as FrameError,
)
from amqp.exceptions import (
    FrameSyntaxError as FrameSyntaxError,
)
from amqp.exceptions import (
    InternalError as InternalError,
)
from amqp.exceptions import (
    InvalidCommand as InvalidCommand,
)
from amqp.exceptions import (
    InvalidPath as InvalidPath,
)
from amqp.exceptions import (
    IrrecoverableChannelError as IrrecoverableChannelError,
)
from amqp.exceptions import (
    IrrecoverableConnectionError as IrrecoverableConnectionError,
)
from amqp.exceptions import (
    NoConsumers as NoConsumers,
)
from amqp.exceptions import (
    NotAllowed as NotAllowed,
)
from amqp.exceptions import (
    NotFound as NotFound,
)
from amqp.exceptions import (
    PreconditionFailed as PreconditionFailed,
)
from amqp.exceptions import (
    RecoverableChannelError as RecoverableChannelError,
)
from amqp.exceptions import (
    RecoverableConnectionError as RecoverableConnectionError,
)
from amqp.exceptions import (
    ResourceError as ResourceError,
)
from amqp.exceptions import (
    ResourceLocked as ResourceLocked,
)
from amqp.exceptions import (
    UnexpectedFrame as UnexpectedFrame,
)
from amqp.exceptions import (
    error_for_code as error_for_code,
)

__all__ = (
    "AMQPError",
    "AMQPNotImplementedError",
    "AccessRefused",
    "Channel",
    "ChannelError",
    "ChannelNotOpen",
    "Connection",
    "ConnectionError",
    "ConnectionForced",
    "ConsumerCancelled",
    "ContentTooLarge",
    "FrameError",
    "FrameSyntaxError",
    "InternalError",
    "InvalidCommand",
    "InvalidPath",
    "IrrecoverableChannelError",
    "IrrecoverableConnectionError",
    "Message",
    "NoConsumers",
    "NotAllowed",
    "NotFound",
    "PreconditionFailed",
    "RecoverableChannelError",
    "RecoverableConnectionError",
    "ResourceError",
    "ResourceLocked",
    "UnexpectedFrame",
    "error_for_code",
    "promise",
)

class promise:
    fun: Callable[..., Any] | None
    args: Sequence[Any]
    kwargs: Mapping[str, Any]
    on_error: Callable[..., Any] | None
    cancelled: bool
    failed: bool
    ready: bool
    reason: BaseException | None
    weak: bool
    ignore_result: bool
    value: Any

    def __init__(
        self,
        fun: Callable[..., Any] | None = ...,
        args: Sequence[Any] | None = ...,
        kwargs: Mapping[str, Any] | None = ...,
        callback: Callable[..., Any] | None = ...,
        on_error: Callable[..., Any] | None = ...,
        weak: bool = ...,
        ignore_result: bool = ...,
    ) -> None: ...
    def __call__(self, *args: Any, **kwargs: Any) -> Any: ...
    def then(
        self, callback: Callable[..., Any], on_error: Callable[..., Any] | None = ...
    ) -> promise: ...
    def throw(
        self,
        exc: BaseException | None = ...,
        tb: Any | None = ...,
        propagate: bool = ...,
    ) -> None: ...
    def throw1(self, exc: BaseException | None = ...) -> None: ...
    def cancel(self) -> None: ...
    @property
    def listeners(self) -> list[promise]: ...



from collections.abc import Callable, Mapping
from types import TracebackType
from typing import Any, TypeVar

_ExcT = TypeVar("_ExcT", bound=BaseException)

def symbol_by_name(
    name: str,
    aliases: Mapping[str, str] | None = ...,
    imp: Callable[[str], Any] | None = ...,
    package: str | None = ...,
    sep: str = ...,
    default: Any = ...,
    **kwargs: Any,
) -> Any: ...
def reraise(tp: type[_ExcT], value: _ExcT, tb: TracebackType | None = ...) -> _ExcT: ...



from collections.abc import Mapping
from datetime import datetime
from typing import Any, TypeVar

import celery.result
import kombu
from celery.app.task import Task
from celery.canvas import Signature
from typing_extensions import ParamSpec

_P = ParamSpec("_P")
_R_co = TypeVar("_R_co", covariant=True)

class DjangoTask(Task[_P, _R_co]):
    def delay_on_commit(
        self, *args: _P.args, **kwargs: _P.kwargs
    ) -> celery.result.AsyncResult[_R_co]: ...
    def apply_async_on_commit(
        self,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        task_id: str | None = ...,
        producer: kombu.Producer | None = ...,
        link: Signature[Any] | list[Signature[Any]] | None = ...,
        link_error: Signature[Any] | list[Signature[Any]] | None = ...,
        shadow: str | None = ...,
        *,
        # options
        countdown: float = ...,
        eta: datetime = ...,
        expires: float | datetime = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: str | kombu.Queue = ...,
        exchange: str | kombu.Exchange = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: dict[str, str] = ...,
        ignore_result: bool = ...,
        time_limit: int = ...,
        soft_time_limit: int = ...,
    ) -> celery.result.AsyncResult[_R_co]: ...



from collections.abc import Callable
from typing import Any, BinaryIO, TextIO

def default_encode(obj: Any) -> bytes: ...
def emergency_dump_state(
    state: Any,
    open_file: Callable[..., BinaryIO | TextIO] = ...,
    dump: Callable[..., None] | None = ...,
    stderr: TextIO | None = ...,
) -> None: ...



from typing import Any

from celery.events.dispatcher import EventDispatcher as EventDispatcher
from celery.events.event import Event as Event
from celery.events.event import get_exchange as get_exchange
from celery.events.event import group_from as group_from
from celery.events.receiver import EventReceiver as EventReceiver

__all__ = (
    "Event",
    "EventDispatcher",
    "EventReceiver",
    "event_exchange",
    "get_exchange",
    "group_from",
)

event_exchange: Any



from typing import Any

__all__ = ("colored",)

class colored:
    def __init__(self, *s: object, **kwargs: Any) -> None: ...
    def __add__(self, other: object) -> str: ...
    def embed(self) -> str: ...
    def no_color(self) -> str: ...
    def node(self, s: tuple[object, ...], op: str) -> colored: ...
    def reset(self, *s: object) -> colored: ...
    def black(self, *s: object) -> colored: ...
    def blink(self, *s: object) -> colored: ...
    def blue(self, *s: object) -> colored: ...
    def bold(self, *s: object) -> colored: ...
    def bright(self, *s: object) -> colored: ...
    def cyan(self, *s: object) -> colored: ...
    def green(self, *s: object) -> colored: ...
    def iblue(self, *s: object) -> colored: ...
    def icyan(self, *s: object) -> colored: ...
    def igreen(self, *s: object) -> colored: ...
    def imagenta(self, *s: object) -> colored: ...
    def ired(self, *s: object) -> colored: ...
    def iwhite(self, *s: object) -> colored: ...
    def iyellow(self, *s: object) -> colored: ...
    def magenta(self, *s: object) -> colored: ...
    def red(self, *s: object) -> colored: ...
    def reverse(self, *s: object) -> colored: ...
    def underline(self, *s: object) -> colored: ...
    def white(self, *s: object) -> colored: ...
    def yellow(self, *s: object) -> colored: ...



from datetime import datetime
from typing import Any

from celery.app.base import Celery

__all__ = ("DjangoFixup", "fixup")

class DjangoFixup:
    app: Celery
    def __init__(self, app: Celery) -> None: ...
    def install(self) -> None: ...
    def autodiscover_tasks(self) -> None: ...
    def now(self, utc: bool = False) -> datetime: ...
    def on_import_modules(self, **kwargs: Any) -> None: ...
    def on_worker_init(self, **kwargs: Any) -> None: ...
    @property
    def worker_fixup(self) -> Any: ...

def fixup(app: Celery, env: str = "DJANGO_SETTINGS_MODULE") -> DjangoFixup | None: ...



import os
from typing import Any

__all__ = ("df", "load_average")

class df:
    def __init__(self, path: str | bytes | os.PathLike[Any]) -> None: ...
    @property
    def available(self) -> float: ...
    @property
    def capacity(self) -> int: ...
    @property
    def stat(self) -> os.statvfs_result: ...
    @property
    def total_blocks(self) -> float: ...

def load_average() -> tuple[float, float, float] | None: ...



# This module requires the optional librabbitmq package
from typing import Any

from kombu.transport.pyamqp import Channel as PyAMQPChannel
from kombu.transport.pyamqp import Connection as PyAMQPConnection
from kombu.transport.pyamqp import Message as PyAMQPMessage
from kombu.transport.pyamqp import Transport as PyAMQPTransport

class Message(PyAMQPMessage): ...
class Channel(PyAMQPChannel): ...

class Connection(PyAMQPConnection):
    Channel: type[Channel]  # pyright: ignore[reportIncompatibleVariableOverride]

class Transport(PyAMQPTransport):
    Connection: type[Connection]  # pyright: ignore[reportIncompatibleVariableOverride]
    default_port: int
    connection_errors: tuple[type[Exception], ...]
    channel_errors: tuple[type[Exception], ...]
    driver_type: str
    driver_name: str
    implements: Any



from collections.abc import Callable, Sequence
from typing import Any, TypeVar, overload

__all__ = ("Signal",)

_F = TypeVar("_F", bound=Callable[..., Any])

class Signal:
    receivers: list[tuple[Any, Any]] | None

    def __init__(
        self,
        providing_args: Sequence[str] | None = None,
        use_caching: bool = False,
        name: str | None = None,
    ) -> None: ...
    @overload
    def connect(
        self,
        receiver: _F,
        sender: Any | None = ...,
        weak: bool = ...,
        dispatch_uid: str = ...,
        retry: bool = ...,
    ) -> _F: ...
    @overload
    def connect(
        self,
        *,
        sender: Any | None = ...,
        weak: bool = ...,
        dispatch_uid: str = ...,
        retry: bool = ...,
    ) -> Callable[[_F], _F]: ...
    def disconnect(
        self,
        receiver: Callable[..., Any] | None = None,
        sender: Any | None = None,
        weak: bool | None = None,
        dispatch_uid: str | None = None,
    ) -> bool: ...
    def has_listeners(self, sender: Any | None = None) -> bool: ...
    def send(
        self, sender: Any | None, **named: Any
    ) -> list[tuple[Callable[..., Any], Any]]: ...
    send_robust = send
